#!/usr/bin/perl
use strict;
use warnings;
use Carp;

# 再帰的にディレクトリを読み込み、svn info を実行する
sub find_max_svn_version() {
	my($path, $max_version) = @_;
	my @files;
	my $file;
	
	opendir(DIR, $path) || die;
	@files = readdir(DIR);
	closedir(DIR);

	foreach $file (@files){
		my($fullpath) = "$path/$file";

		if (!($fullpath =~ /([^\/]*[^\/]*$)/ && $1 =~ /^\..*/ )) {
			if (-d $fullpath) {
				print "Searching Directory: $fullpath\n";
				$max_version = &find_max_svn_version($fullpath, $max_version);
			} else {
				my $temp = get_svn_version($fullpath);
				if ($temp =~ /\d+/) {
					# max SVNバージョンが更新される場合
					if ($temp > $max_version) { 
						printf "New Revision Found:$fullpath $max_version　-> $temp\n";
						$max_version = $temp; 
					}
				}
			}
		}
	}
	return $max_version;
}

# svn info からRevisionを取り出す
sub get_svn_version() {
	my($target) = @_;

	open(FIN, "LANG=C NOLOCATE=1 svn info $target 2> /dev/null |");
	my @line = <FIN>;
	close(FIN);
	
	my $find = 0;
	my $version = "";
	foreach my $line (@line) {
		if ($line =~ /Last\s+Changed\s+Rev:\s*(\d+)/) {
			# print "$target Revision = $1\n";
			$find = 1;
			$version = $1;
			last;
		}
	}
	return $version; # 
}

# コマンド引数
my %replaces=();
my @targets=();
my $svn_key="";

sub main() {

# ヘルプの表示
	if ($#ARGV == 0 && ($ARGV[0] eq "--help" || $ARGV[0] eq "-h")) {
print <<"EOF";
Usage: 
$0 TARGET [...] KEY=VALUE [...] [--svn=SVN_KEY]
$0 --printsvn

Description:
$0 script replaces "KEY=***" string with "KEY=VLAUE" string in TARGET file. 
You can specify arbitrary string for *** and the string must be put around double quotation.
If KEY sames SVN_KEY, it replaces its VALUE string with SVN revision in current directory.
SVN revision is max number to recursively-search directories.
Then its VALUE is ignored.
If --printsvn argument is specified, it will display only SVN revision to standard output.
EOF
		exit;
	}

	# コマンド引数の取得
	my $find_target = 0;
	my $printsvn = 0;
	foreach my $arg (@ARGV) {
		if ($arg =~ /^([^-]+)=(.*)/) {
			$replaces{$1} = $2;
		} elsif ($arg =~ /^--svn=(.+)/) {
			$svn_key = $1;
		} elsif ($arg =~ /^--printsvn$/) {
			$printsvn = 1;
		} else {
			#if ($find_target) {
			#	print "Target file is multiple defined.";
			#	exit;
			#}
			push(@targets, $arg)
			# $find_target = 1;
		}
	}

	# 引数のエラーチェック
	if ($printsvn == 1 && $#ARGV != 0) {
		croak "--svnprint argument is enable but other arument exists.\n";
	}
	if ($printsvn != 1) {
		if (! ($#targets +1) ) {
			croak "Can't find target file.\n";
		}
		if (! keys %replaces) {
			croak "Cant't find replacing keyword.\n";
		}
		if ($svn_key ne "" && !exists $replaces{$svn_key}) {
			croak "svn's keyword can't found in replacing argument\n";
		}
	}

	# SVN バージョンだけを出力
	if ($printsvn) {
		open COPY , '>&STDOUT';
		open STDOUT , '>/dev/null';
		my $revision = &find_max_svn_version(".", -1);
		if ($revision > 0) {
			print COPY "$revision";
			close(COPY);
			exit 0;
		} else {
			close(COPY);
			exit 1;
		}
	}


	# SVN Revision の取得
	my $max_version = -1; # バージョンの初期値は無効な-1にしておく
	if ($svn_key ne "") {
		if (system("which svn") != 0) {
			carp "Warning: svn command can't found.\n";
		} else {
			$max_version = &find_max_svn_version(".", -1);
		}
	}

	# バージョン文字列の置換
	foreach my $target (@targets) {

		# ファイルの読み書きチェック
		if (!(-r $target && -w $target)) {
			croak "Can't read and write file:$target.\n";
		}

		open(FIN, $target);
		my @lines;
		while(<FIN>) {
			while ((my $key, my $value)= each %replaces) {
				if ($key eq $svn_key) {
					if ($max_version >= 0) {
						$value = "$max_version";
					} else {
						$value = ""; # リビジョンが見つからない場合は強制的に空文字にする。 
					}
				}
				# キーワードに一致する行を置換する
				if ($_ =~ s/(([^\\]|^)"$key=)([^"\\]*)"/$1$value"/) {
					print "$target: Keyword Matched Line: $_";
				}
			}
			push(@lines, $_);
		}
		close(FIN);

		# 置換されたファイルの書き込み
		open(FOUT, ">$target");
		foreach my $line (@lines) {
			print FOUT $line;
		}
		close(FOUT);
	}
}

&main();


